# -*- coding: utf-8 -*-
"""
Created on Tue Nov 18 11:19:37 2014

@author: BRAUOLI
"""
from __future__ import print_function
from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto import Random
#base64 is used for encoding. dont confuse encoding with encryption#
#encryption is used for disguising data
#encoding is used for putting data in a specific format
import base64, os, binascii
# os is for urandom, which is an accepted producer of randomness that
# is suitable for cryptology.
#import os

def read_from_file(filename):
    with open(filename, "rb") as in_file:
        data = in_file.read()
    return data

def write_to_file(filename, data, chunksize=8192):
    """
    Write bytes from data into file
    
    :param filename: name of file
    :param data: list with data (turned into bytearrays)
    :param chunksize: the bit-size which is transmitted at once
    """
    with open(filename, "wb") as f:
        loc = 0
        length = len(data)
        while True:
            if loc+chunksize < length:
                #print filename, loc, length
                chunk=data[loc:loc+chunksize]
                f.write(chunk)
                loc += chunksize
            else:
                chunk=data[loc:length]
                f.write(chunk)
                break
            
def get_crc_from_file(filename):
    if os.path.exists(filename):
        data = read_from_file(filename) #filename includes the path of the origin
        return binascii.crc32(data)
    
def random(n):
    out = Random.new()
    number = out.read(n)
    hexnumber = binascii.hexlify(number)
    return hexnumber

class CryptoAESBase():
    """
    class is based on bytes as input and output. Nevertheless tolerant against unicode and string input.
    """
    def __init__(self, passwd):
        if not type(passwd) is bytes:
            self.passwd = passwd.encode()
        else:
            self.passwd = passwd
        myhash = SHA256.new()
        myhash.update(self.passwd)
        key = myhash.digest()
        self.cipher = AES.new(key)
        self.PADDING = b"{"
        
    def encryption(self,pI):
        # the block size for cipher obj, can be 16 24 or 32. 16 matches 128 bit.
        BLOCK_SIZE = 16
        if not type(pI) is bytes:
            unipI = pI.encode()
        else:
            unipI = pI
            
        self.pad = lambda s: s + (BLOCK_SIZE - len(s) % BLOCK_SIZE) * self.PADDING
        # encrypt with AES, encode with base64
        EncodeAES = lambda c, s: base64.b64encode(c.encrypt(self.pad(s)))
        
        encoded = EncodeAES(self.cipher, unipI)
        return encoded
    
    def decryption(self,encryptedBytes):
        DecodeAES = lambda c, e: c.decrypt(base64.b64decode(e)).rstrip(self.PADDING)
 
        try:
            decodedBytes = DecodeAES(self.cipher, encryptedBytes)
        except:
            decoded = encryptedBytes
            #print("seems not to be encrypted!", decoded)

        return decodedBytes




from __future__ import print_function
from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA
from Crypto import Random
import binascii, os, sys

#_path_ = os.path.dirname(os.path.abspath(__file__))
_path_ = os.getcwd()

ispython3 = sys.version_info > (3, 0)

if ispython3:
    def unicoding(x):
        #print(type(x),x[0])
        if type(x) is bytearray:
            return x.decode()
        elif type(x) is bytes:
            return x.decode()
        elif type(x) is str:
            return x
        else:
            return x
               
else: 
    def unicoding(x):
        if type(x) is bytearray:
            return unicode(x) #better use decode here?
        elif type(x) is bytes:
            return unicode(x) #better use decode here?
        elif type(x) is str:
            x = x.decode()
            return x
        elif type(x) is unicode:
            return x
        else:
            return x


class CryptoRSABase():
    """
    Class is based on bytes as in- and output. Nevertheless tolerant against unicode and string input.
    """
    fullkey = None
    extkey = None
    path = os.path.join(_path_,"PEM")
    filename = 'localkey.pem'
    
    def create_rsa_key(self, name):
        self.filename = ''.join([self.path,"/",name,'_localkey.pem'])
        self.fullkey = key = RSA.generate(2048)
        if not os.path.exists(self.path):
            os.mkdir(self.path)
        with open(self.filename,'w') as f:
            f.write(unicoding(key.exportKey('PEM')))
        return key

    def get_key_by_file(self, name):
        self.filename = ''.join([self.path,"/",name,'_localkey.pem'])
        if not os.path.exists(self.filename):
            self.create_rsa_key(name)
        with open(self.filename, 'r') as f:
            key_txt = f.read()
        self.fullkey = key = RSA.importKey(key_txt)
        return key

    def get_hex_key(self, name):
        if not self.fullkey:
            self.get_key_by_file(name)
        return unicoding(binascii.hexlify(self.fullkey.publickey().exportKey('DER')))
        
    def random(self,n):
        out = Random.new()
        number = out.read(n)
        hexnumber = binascii.hexlify(number)
        return hexnumber

    def encrypt(self,message,hexkey):
        if not type(message) is bytes:
            message = message.encode()

        h = SHA.new(message)

        key = RSA.importKey(binascii.unhexlify(hexkey))
        cipher = PKCS1_v1_5.new(key)
        tail = h.digest()
        return cipher.encrypt(message+tail)

    def decrypt(self,name,ciphertext):
        if not self.fullkey:
            self.get_key_by_file(name)
        cipher = PKCS1_v1_5.new(self.fullkey)
        dsize = SHA.digest_size
        sentinel = Random.new().read(20+dsize)      # Let's assume that average data length is 15
        message = cipher.decrypt(ciphertext, sentinel)
        digest = SHA.new(message[:-dsize]).digest()
        if digest==message[-dsize:]:                # Note how we DO NOT look for the sentinel
            #print ("Encryption was correct..",message[:-dsize])
            return message[:-dsize]
        else:
            #print ("Encryption was not correct.")
            return message[:-dsize]
        
